*                                    AVERTISSEMENT

* Le gouvernement du Canada (Statistique Canada) possde tous les droits de proprit
* intellectuelle (y compris le droit d'auteur) relatifs  ce logiciel.  Sous rserve
* des modalits dcrites ci-dessous, vous obtenez une licence non exclusive et non
* transmissible d'utilisation de ce logiciel.
*
* Ce logiciel vous est fourni  tel quel , et le titulaire ne donne aucune garantie
* expresse ou implicite, ce qui comprend les garanties de qualit marchande et de
* convenance  une fin particulire mais ne se limite pas  ces garanties.  En aucune
* circonstance le titulaire ne sera responsable de dommages indirects, particuliers ou
* conscutifs, ou d'autres dommages semblables.  Ce contrat sera automatiquement rsili,
* sans pravis, si vous omettez de respecter une quelconque des modalits du contrat.;

/*******************************************************************/
/* Date: Avril 2002                                                */
/* Modifi: Avril 2002                                             */
/*******************************************************************/

/*********************************************************************************/
/***                                                                           ***/
/***                             MACROF_V20.SAS                                ***/
/***                                                                           ***/
/*** Ce programme calcule des estimations de variance pour diffrents types    ***/
/*** d'estimations  l'aide des poids bootstrap.   l'aide de macros SAS,      ***/
/*** ce programme peut calculer des estimations de variance pour des totaux,   ***/
/*** des rapports, et des diffrences de rapports.  Il peut aussi calculer des ***/
/*** estimations de variance pour des paramtres de rgressions linaires ou   ***/
/*** logistiques.  Ce programme peut aussi tre modifi pour s'adapter        ***/
/*** d'autres types d'analyse.                                                 ***/
/***                                                                           ***/
/*** Ce programme contient les macros qui sont ncessaires pour utiliser le    ***/
/*** programme BOOTVARF_V20.SAS.                                               ***/
/***                                                                           ***/
/*** Ce programme est appel automatiquement par BOOTVARF_V20.SAS et AUCUNE    ***/
/*** MODIFICATION NE DEVRAIT ETRE APPORTE PAR L'UTILISATEUR (sauf pour        ***/
/*** quelques cas prcis mentionns dans BOOTVARF.SAS)                         ***/
/***                                                                           ***/
/*********************************************************************************/

options ps=48 ls=120 nonotes;


/*********************************************************************************/
/*** Section 1: Dclaration des variables macros                               ***/
/*********************************************************************************/

data time;
  format debut datetime16.;
  debut=datetime();
  output;
run;

* VARIABLES SPCIFIQUES A CHAQUE VERSION DE SAS:     ;

data _NULL_;
if &sysver >= 8 then call symput ('inter' ,'intercept');
else call symput ('inter' ,'intercep');
run;

* VARIABLES POUR L'IDENTIFICATION DES RPONDANTS ET POUR L'INDEX(section suivante):   ;


data _NULL_;
set &Mfile;
       if realukey ne '' then do; 
                                     call symput ('ident' ,'realukey personid');
                                     call symput ('indx','(id=(realukey personid))'); 
                             end;
       else if uniqueid ne '' then do;   
                                      call symput ('ident' ,'uniqueid');
                                      call symput ('indx','(uniqueid)');  
                                  end;
       else if sampleid ne '' then do;   
                                     call symput ('ident' ,'sampleid personid');
                                     call symput ('indx','(id=(sampleid personid))'); 
                                   end;
       else do;
       	  put " ERREUR:  Les variables d'identification des rpondants requises n'ont  ";
	      put " pas t trouves...";
       	  put " ";
       	  put " Pour l'ENSP, composante mnage, ces variables sont: REALUKEY et PERSONID";
       	  put " Pour l'ENSP, composante tablissement de sant, la variable est: UNIQUEID";
       	  put " Pour l'ESCC, ces variables sont: SAMPLEID et PERSONID";
     	  put " ";
       	  put " Veuillez vrifier que ces variables sont prsentes dans le fichier d'analyse";
       	  put " (utiliser la fonction RENAME si les noms de variables ne correspondent pas)";
	  end;
run;

OPTIONS notes;


***************************************************************************;
*  Section 2: LECTURE DU FICHIER PRINCIPAL ET APPARIEMENT AVEC LES POIDS  *;
***************************************************************************;

data Mfile (index=&indx);
   set &Mfile ;
run;

/* Lit les poids de bootstrap */
/* FWGT est le mme poids celui inclus dans le fichier d'analyse */

data bsamp (index=&indx);
  set &bsamp;
  keep fwgt &ident bsw1-bsw&b;
run;

/* Apparie le fichier principal et les poids bootstrap */

     data bs_data ;
       merge Mfile (in=in1) bsamp (in=in2);
       by &ident;
       if in1;
     run;

/*  Vrification si variable(s) de ventilation */ 

%let by=;
%let cla_tmp="&classes";

data _NULL_;
if substr(&cla_tmp,1,1)='.' then do;
                   call symput ('by' ,'*');
                   call symput ('classes' ,'');
				   end;
run;

&by  proc sort data=bs_data;
&by     by &classes;
&by  run;


proc datasets library=work;
      delete Mfile bsamp version;
run;

* FICHIERS DE RSULTATS:     ;

data alltots diffrat bs_reg bs_reglg ;
      set _NULL_;
run;

%let result= alltots diffrat bs_reg bs_reglg;


/*********************************************************************************/
/*** Section 3: Dclaration des macros                                         ***/
/*********************************************************************************/

%let printtot=0;
%let printdif=0;
%let printreg=0;
%let printlog=0;

%global dep1 dep2;

*************************************;

%macro total(var);

*************************************;

proc means data=bs_data noprint;
  var fwgt bsw1-bsw&b;
  weight &var;
&by  by &classes;
  output out=ytot
         sum=yhat ybs1-ybs&b;
run;

data est;
  set ytot;
  length var1 $ 8;
  length var2 $ 8;
  length type $ 8;
  bs_var=((&b-1)*(var(of ybs1-ybs&b)))/&b;
  bs_sd=sqrt(bs_var);
  bs_cv=round((bs_sd/yhat)*100,.01);
  cil95=yhat-1.96*bs_sd;
  ciu95=yhat+1.96*bs_sd;
  var1="&var";
  var2="Aucune";
  type="Total";
  drop ybs1-ybs&b _type_ _freq_;
run;

data alltots;
	set alltots est;
run;

%let printtot=1;

proc datasets library=work;
	delete ytot est;
run;

%mend total;


*******************************************;

%macro ratio(var1,var2,);

*******************************************;

proc means data=bs_data noprint;
  var fwgt bsw1-bsw&b;
  weight &var1;
&by  by &classes;
  output out=ytot
         sum=yhat ybs1-ybs&b;
run;

proc means data=bs_data noprint;
  var fwgt bsw1-bsw&b;
  weight &var2;
  &by  by &classes;
  output out=xtot
  sum=xhat xbs1-xbs&b;
run;

data est;
  merge ytot xtot;
  array ybs{&b};
  array xbs{&b};
  array rbs{&b};
  length var1 $ 8;
  length var2 $ 8;
  length type $ 8;
  yhat=((yhat/xhat)*100);
  do i=1 to &b;
    rbs{i}=((ybs{i}/xbs{i})*100);
  end;
  bs_var=((&b-1)*(var(of rbs1-rbs&b)))/&b;
  bs_sd=sqrt(bs_var);
  bs_cv=round((bs_sd/yhat)*100,.01);
  cil95=yhat-1.96*bs_sd;
  ciu95=yhat+1.96*bs_sd;
  var1="&var1";
  var2="&var2";
  type="Rapport";
  drop ybs1-ybs&b xbs1-xbs&b rbs1-rbs&b xhat i _type_ _freq_;
run;

data alltots;
    set alltots est;
run;

%let printtot=1;

proc datasets library=work;
    delete ytot xtot est;
run;

%mend ratio;


********************************************************;

%macro diff_rat(var1,var2,var3,var4);

********************************************************;

proc means data=bs_data noprint;
  var fwgt bsw1-bsw&b;
  weight &var1;
  &by  by &classes;
  output out=ytot
  sum=yhat ybs1-ybs&b;
run;

proc means data=bs_data noprint;
  var fwgt bsw1-bsw&b;
  weight &var2;
  &by  by &classes;
  output out=xtot
  sum=xhat xbs1-xbs&b;
run;

proc means data=bs_data noprint;
  var fwgt bsw1-bsw&b;
  weight &var3;
  &by  by &classes;
  output out=yytot
  sum=yyhat yybs1-yybs&b;
run;

proc means data=bs_data noprint;
  var fwgt bsw1-bsw&b;
  weight &var4;
  &by  by &classes;
  output out=xxtot
  sum=xxhat xxbs1-xxbs&b;
run;

data est;
  merge ytot xtot yytot xxtot;
  array ybs{&b};
  array xbs{&b};
  array yybs{&b};
  array xxbs{&b};
  array drbs{&b};
  length var1 $ 8;
  length var2 $ 8;
  length var3 $ 8;
  length var4 $ 8;
  length type $ 10;
  yhat=(((yhat/xhat)-(yyhat/xxhat))*100);
  do i=1 to &b;
    drbs{i}=(((ybs{i}/xbs{i})-(yybs{i}/xxbs{i}))*100);
  end;
  bs_var=(((&b-1)*(var(of drbs1-drbs&b)))/&b);
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/yhat)*100,.01));
  cil95=yhat-1.96*bs_sd;
  ciu95=yhat+1.96*bs_sd;
  var1="&var1";
  var2="&var2";
  var3="&var3";
  var4="&var4";
  type="Diff_Rapport";
  drop ybs1-ybs&b xbs1-xbs&b yybs1-yybs&b xxbs1-xxbs&b drbs1-drbs&b
       xhat xxhat yyhat i _type_ _freq_;
run;

data diffrat;
   set diffrat est;
run;

%let printdif=1;

proc datasets library=work;
   delete ytot xtot yytot xxtot est;
run;

%mend diff_rat;


*********************************************;

%macro regress(yvar,xvar);

*********************************************;

proc reg data=bs_data outest=orig(keep=&classes &inter &xvar) noprint;
  model &yvar=&xvar;
  weight fwgt;
  &by  by &classes;
run;

proc transpose data=orig out=origest(drop=_label_) prefix=bhat name=beta;
  var &inter &xvar;
  &by  by &classes;
run;

data _NULL_; 
    L=int((&b/10)+0.999);
    call symput ('L' , trim(left(L))); 
run;

OPTIONS nonotes;

%let j_dep=1;

%do k=1 %to 10;
  %let j=%eval(1+((&k-1)*&L));
  %let kL=%eval(&k*&L);

data _NULL_;
if (&b - &kL) >0  then do;   
                         k=&k;
                         j=1+((&k-1)*&L);
                         kL=&k*&L;
                       end;
                  else do; 
                         k=10; 
                         j=&j_dep;
                         kL=&b;
                       end;

call symput ('k' , trim(left(k)));
call symput ('j' , trim(left(j)));
call symput ('kL' , trim(left(kL)));
run;

data poids (keep = bsw&j-bsw&kL &yvar &xvar &classes);  
  set bs_data;
run;

  %do i=&j %to &kL;

   %put Regression &i termine;

   %let j_dep=%eval(&kl+1);

   proc reg data=poids outest=betas(keep=&classes &inter &xvar) noprint;
     model &yvar=&xvar;
     weight bsw&i;
&by  by &classes;
   run;

   proc transpose data=betas out=betat prefix=best name=beta;
     var &inter &xvar;
&by  by &classes;
   run;

   data betat;
     set betat;
     drop _label_;
     rename best1=best&i;
	if &i=1 then test=1;
	else test=test+1;
   run;

   %if (&i =1) %then %do;

     data bsbeta;
       set betat;
     run;

   %end;
   %else %do;

     data bsbeta;
       merge bsbeta betat;
&by    by &classes;
     run;

   %end;
  %end;
%end;

OPTIONS notes;

data est;
  merge origest bsbeta;
  rename bhat1=bhat;
  bs_var=((&b-1)*(var(of best1-best&b)))/&b;
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/bhat1)*100,.01));
  cil95=bhat1-1.96*bs_sd;
  ciu95=bhat1+1.96*bs_sd;
  ydep="&yvar";
  drop best1-best&b;
run;

data bs_reg;
    set bs_reg est;
run;

%let printreg=1;
%let dep1=&yvar;

proc datasets library=work;
    delete betas betat bsbeta origest;
run;


%mend regress;

*********************************************;

%macro logreg(yvar,xvar);

*********************************************;

proc logistic data=bs_data outest=orig(keep=&classes &inter &xvar) descending noprint;
  model &yvar=&xvar;
  &by by &classes;
  weight fwgt;
run;

proc transpose data=orig out=origest prefix=bhat name=beta;
  var &inter &xvar;
  &by by &classes;
run;


data _NULL_; 
    L=int((&b/10)+0.999);
    call symput ('L' , trim(left(L))); 
run;

OPTIONS nonotes;

%let j_dep=1;

%do k=1 %to 10;
  %let j=%eval(1+((&k-1)*&L));
  %let kL=%eval(&k*&L);

data _NULL_;
if (&b - &kL) >0  then do; 
                          k=&k;
                          j=1+((&k-1)*&L);
                          kL=&k*&L;
                       end;
                  else do; 
                          k=10; 
                          j=&j_dep;
                          kL=&b;
                       end;

call symput ('k' , trim(left(k)));
call symput ('j' , trim(left(j)));
call symput ('kL' , trim(left(kL)));
run;


data poids (keep = bsw&j-bsw&kL &yvar &xvar &classes);  
  set bs_data;
run;

  %do i=&j %to &kL;

  %put Regression logistique &i termine;
 
    %let j_dep=%eval(&kl+1);

    proc logistic data=poids outest=betas (keep=&classes &inter &xvar) noprint descending;
      model &yvar=&xvar;
      &by by &classes;
      weight bsw&i;
    run;



    proc transpose data=betas out=betat prefix=best name=beta;
      var &inter &xvar;
      &by by &classes;
    run;

    data betat;
      set betat;
      rename best1=best&i;
    run;

    %if (&i =1) %then %do;

      data bsbeta;
        set betat;
      run;

    %end;
    %else %do;

      data bsbeta;
        merge bsbeta betat;
        &by by &classes;
      run;

    %end;
  %end;
%end;

OPTIONS notes;

data est;
  merge origest bsbeta;
  rename bhat1=bhat;
  bs_var=((&b-1)*(var(of best1-best&b)))/&b;
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/bhat1)*100,.01));
  wald=(bhat1/bs_sd)*(bhat1/bs_sd);
  pvalue=1-probchi(wald,1);
  lo95=bhat1-1.96*bs_sd;
  hi95=bhat1+1.96*bs_sd;
  odds=exp(bhat1);
  cil95=exp(lo95);
  ciu95=exp(hi95);
  ydep="&yvar";
  drop best1-best&b;
run;

data bs_reglg; 
     set bs_reglg est;
run;

%let printlog=1;
%let dep2=&yvar;

proc datasets library=work;
     delete betas betat bsbeta origest;
run;


%mend logreg;



****************;

%macro prnttot;

****************;
%if &printtot=1 %then %do;

&by   proc sort data=alltots;
&by     by &classes;
&by   run;

/**************************************************/
/* Imprime les rsultats des macro total et ratio */
/**************************************************/

proc print data=alltots;
    	 title "Estimation de la variance  l'aide du bootstrap &B pour des";
    	 title2 "Totaux et des Rapports";
	 title3 " ";
	 var &classes type var1 var2 yhat bs_sd bs_cv cil95 ciu95;
  	 format yhat bs_sd cil95 ciu95 11.2;
run;

%end;

        /**********************************************************/
        /*** O:                                                ***/
        /*** type         : type d'estimation (Total ou Rapport)***/
        /*** var1 et var2 : variables utilises pour calculer   ***/
        /***                les estimations.  Pour un total,    ***/
        /***                var2=Aucune.                        ***/
        /*** yhat         : estimation du paramtre (en % pour  ***/
        /***                un rapport)                         ***/
        /*** bs_sd        : cart-type                          ***/
        /*** bs_cv        : coefficient de variation            ***/
        /*** cil95        : borne infrieure de l'intervalle de ***/
        /***                confiance  95%                     ***/
        /*** ciu95        : borne suprieure de l'intervalle de ***/
        /***                confiance  95%                     ***/
        /**********************************************************/


%mend prnttot;

****************;

%macro prntdiff;

****************;
%if &printdif=1 %then %do;

&by   proc sort data=diffrat;
&by     by &classes;
&by   run;



/**********************************************/
/* Imprime les rsultats de la macro diff_rat */
/**********************************************/

   proc print data=diffrat;
     title "Estimation de la variance  l'aide du bootstrap &B pour des";
     title2 "Diffrences entre des Rapports";
     title3 "";
     var &classes type var1 var2 var3 var4 yhat bs_sd bs_cv cil95 ciu95;
     format yhat bs_sd cil95 ciu95 11.2;
   run;
%end;

        /**********************************************************/
        /*** O:                                                ***/
        /*** type         : type d'estimation (Diff_Rapport)    ***/
        /*** var1,var2,                                         ***/
        /*** var3 et var4 : variables utilises pour calculer   ***/
        /***                l'estimation                        ***/
        /*** yhat         : estimation (diffrence en %)        ***/
        /*** bs_sd        : cart-type                          ***/
        /*** bs_cv        : coefficient de variation            ***/
        /*** cil95        : borne infrieure de l'intervalle de ***/
        /***                confiance  95%                     ***/
        /*** ciu95        : borne suprieure de l'intervalle de ***/
        /***                confiance  95%                     ***/
        /**********************************************************/



%mend prntdiff;

****************;

%macro prntreg;

****************;
%if &printreg=1 %then %do;


/******************************************************/
/* Imprime les rsultats de la macro regress          */
/******************************************************/

   proc print data=bs_reg;
     title "Estimation de la variance  l'aide du bootstrap &B pour une";
     title2 "Rgression";
     title3 "Variable dpendante: &dep1";
     var &classes beta bhat bs_var bs_sd bs_cv cil95 ciu95;
   run;
%end;

        /**********************************************************/
        /*** O:                                                ***/
        /***   beta       : paramtre  estimer                 ***/
        /***   bhat       : estimation du paramtre             ***/
        /***   bsvar      : variance de l'estimation du paramtre */
        /***   bs_sd      : cart-type de l'estimation du       ***/
        /***                paramtre                           ***/
        /***   bs_cv      : coefficient de variation de l'esti- ***/
        /***                mation du paramtre                 ***/
        /***   cil95      : borne infrieure de l'intervalle de ***/
        /***                confiance  95%                     ***/
        /***   ciu95      : borne suprieure de l'intervalle de ***/
        /***                confiance  95%                     ***/
        /**********************************************************/


%mend prntreg;

****************;

%macro prntlog;

****************;
%if &printlog=1 %then %do;

/********************************************/
/* Imprime les rsultats de la macro logreg */
/********************************************/

   proc print data=bs_reglg;
     title "Estimation de la variance  l'aide du bootstrap &B pour des";
     title2 "Rgressions logistiques";
     title3 "Variable dpendante: &dep2";
         var  &classes beta bhat odds wald pvalue bs_var bs_sd bs_cv cil95 ciu95;
	 run;
%end;

        /**********************************************************/
        /*** O:                                                ***/
        /***   beta       : paramtre  estimer                 ***/
        /***   bhat       : estimation du paramtre             ***/
        /***   odds       : rapport de cotes                    ***/
        /***   wald       : statistique de Wald                 ***/
        /***   pvalue     : valeur p de la statistique de Wald  ***/
        /***   bsvar      : variance de l'estimation du paramtre */
        /***   bs_sd      : cart-type de l'estimation du       ***/
        /***                paramtre                           ***/
        /***   bs_cv      : coefficient de variation de         ***/
        /***                l'estimation du paramtre           ***/
        /***   cil95      : borne infrieure de l'intervalle de ***/
        /***                confiance  95% du rapport de cotes ***/
        /***   ciu95      : borne suprieure de l'intervalle de ***/
        /***                confiance  95% du rapport de cotes ***/
        /**********************************************************/


%mend prntlog;



****************;

%macro output;

****************;

	%prnttot;
	%prntdiff;
	%prntreg;
	%prntlog;

	data time;
 	 set time;
	  format fin datetime16.;
 	 fin=datetime();
	  output;
	run;

	proc print data=time;
  		title 'Temps requis pour excuter le programme';
	run;

%mend output;



/* Fin du programme SAS MACRO.SAS */
